local shaders = require 'graphics.shaders'

local draw = {}

local resize = false

function draw.load(gameState)
  shaders.load(gameState)
  local w, h, flags = love.window.getMode()
  flags.resizable = true
  love.window.setMode(w, h, flags)
end

function draw.update(gameState)
  shaders.update(gameState)
end

function draw.draw(gameState)
  if resize then
    shaders.resize(gameState, w, h)
    resize = false
  end

  local window = {
    w = love.graphics.getWidth(),
    h = love.graphics.getHeight(),
  }

  -- shadows needs color to shade
  love.graphics.rectangle('fill', 0, 0, window.w, window.h)
  -- shaders are offset already in update
  shaders.draw(gameState)

  if gameState.physics.player then
    love.graphics.push()
    local camera = {
      x = -1*gameState.physics.player:getX() + window.w/2,
      y = -1*gameState.physics.player:getY() + window.h/2,
    }
    -- offset for drawing the world
    love.graphics.translate(camera.x, camera.y)
    gameState.physics.world:draw()
    love.graphics.pop()
  end

  if gameState.fsm.state == "load" then
    love.graphics.print("Loading", 10, 10)
  else
    love.graphics.print(("FPS: " .. tostring(love.timer.getFPS())), 10, 10)
  end
end

function draw.window(gameState)
  if love.window.getTitle() ~= gameState.title then
    love.window.setTitle(gameState.title)
  end
end

function love.resize(w, h)
  -- resize on next frame when gameState is in scope
  resize = true
end

return draw
