local shaders = {}

-- TODO pull request a require fix so shadows works from the lib/ folder
-- load in modules from shadows
local Shadows = require("shadows")
-- stars seem to have way better performance than lights
-- this seems to be implied in the wiki as the star is created for thousands
-- of lighting pixels and a light for hundreds
local Star = require("shadows.Star")
local LightWorld = require("shadows.LightWorld")
local Body = require("shadows.Body")

local function attatchLight(world, body, light)
  light:GetTransform():SetParent(world:TrackBody(body))
end

function shaders.load(gameState)
  gameState.shaders = {}
  gameState.shaders.world = LightWorld:new()
  gameState.shaders.bodies = {}
  -- listen to the creation / removal of bodies
  -- to sync to shaders objects
  local new = gameState.physics.listeners.new
  new[#new + 1] = function(gameState, object, message)
    local world = gameState.shaders.world
    if message == "player" then
      gameState.shaders.player = Star:new(world, 500):SetColor(255, 0, 0)
      attatchLight(world, object, gameState.shaders.player)
    end
    if message == "terrain" then
      local bodies = gameState.shaders.bodies
      -- refactor logic into helpers
      --local body = Body:new(gameState.shaders):TrackPhysics(object)
      --PolygonShadow:new(body, object:getPoints())
      local body = Body:new(world)
      body:InitFromPhysics(object)
      bodies[#bodies + 1] = body
    end
  end
  local remove = gameState.physics.listeners.remove
  remove[#remove + 1] = function(gameState, object, message)
    if message == "player" then
      gameState.shaders.player:Remove()
      gameState.shaders.player = nil
    end
  end
  local reset = gameState.physics.listeners.reset
  remove[#remove + 1] = function(gameState, object, message)
    if message == "player" then
      gameState.shaders.player:Remove()
      gameState.shaders.player = nil
    end
  end
end


function shaders.draw(gameState)
  gameState.shaders.world:Draw()
end

function shaders.update(gameState)
  local world = gameState.shaders.world
  if gameState.physics.player then
    local player = {
      x = gameState.physics.player:getX(),
      y = gameState.physics.player:getY(),
    }
    local window = {
      w = love.graphics.getWidth(),
      h = love.graphics.getHeight(),
    }
    -- center the shaders on the player position
    world:SetPosition(-window.w/2 + player.x, -window.h/2 + player.y)
  else
    world:SetPosition(0, 0)
  end
  world:Update()
end

function shaders.resize(gameState, w, h)
  gameState.shaders.world:Resize(w, h)
end

return shaders
