local world = require 'logic.world.world'

-- a minimal finite state machine to
-- track game state transitions
-- this is not a DFA as the equivalent of epsilon
-- transitions are used and most states do
-- not have a full set of transitions defined
local fsm = {}

local FSM = {}

local FSM = {
  update = true,
  -- define the starting state of the fsm
  state = "load",
  states = {
    load = function(gameState)
      -- load game state should immediately transition to the menu
      -- this enforces that all code waits 1 tick for the menu
      -- game state to be loaded and should prevent code assuming
      -- the gameState has properties it won't at particular times
      gameState.fsm.state = "menu"
    end,
    menu = function(gameState)
      world.reload(gameState)
      -- do not immediately change the game state when menu
    end
  }
}

function fsm.load(gameState)
  gameState.fsm = FSM
end

function fsm.update(gameState)
  -- only trigger transition function if just changed state
  if gameState.fsm.update then
    current = gameState.fsm.state
    gameState.fsm.states[current](gameState)
    -- only trigger the transition function next update
    -- if the current state has changed
    gameState.fsm.update = current ~= gameState.fsm.state
  end
end

return fsm
