local draw = require 'graphics.draw'
local update = require 'logic.update'
local input = require 'input.input'
-- debugging only
inspect = require 'lib.inspect.inspect'

-- the game state
-- everything added to the game state in a module's load() function
-- can always be assumed to exist here during updates or drawing
-- anything else may not be present
-- in most cases modules should only access parts of the game state
-- they have added, with the obvious exception for drawing code
-- which creates nothing and accesses almost everything - but should
-- not mutate anything it accesses
local gameState

-- track time steps to take
local timestep
-- length of one time step in seconds
local TIME_STEP = 1/60

function love.load()
  gameState = {
    time = 0,
    inputs = input.load(),
  }
  timestep = 0
  update.load(gameState)
  draw.load(gameState)
end

function love.update(dt)
  input.collect(gameState.inputs)
  timestep = timestep + dt
  while timestep > TIME_STEP do
    -- increase time passed by 1 time step
    gameState.time = gameState.time + TIME_STEP
    -- update the game state by 1 time step
    timestep = timestep - TIME_STEP
    update.update(gameState)
    -- discard unclaimed inputs
    input.discard(gameState.inputs)
  end
  draw.update(gameState)
end

function love.draw()
  draw.draw(gameState)
  draw.window(gameState)
end
